/*****************************************************************************
 * Orangutan SV-168 and LV-168 project 2:                                    *
 * User buttons and advanced buzzer control (using hardware PWM).            *
 *****************************************************************************/

// This program uses one of the mega168's hardware PWMs to play the buzzer
// while the user buttons are held down.  The button being held determines the
// frequency of the note the buzzer is playing.  This program does NOT use
// the Pololu AVR library (libpololu).


#include <avr/io.h>			// provides AVR port and register #defines

#define F_CPU 20000000UL	// the Orangutan xV-168 has a clock speed of 20 MHz
#include <util/delay.h>		// uses F_CPU to provide delay routines


// ***** BUTTON PINS *****
#define BUTTON0		PB1		// pin for bottom user button
#define BUTTON1		PB4		// pin for middle user button
#define BUTTON2		PB5		// pin for top user button



int main()
{
	// The user buttons connect the button IO pins to ground through a 1k
	//  resistor when the buttons are pressed.  Therefore, to make use of
	//  the buttons, you should set the button pins as inputs, enable their
	//  internal pull-ups, and monitor the pin value to see when it goes low.

	//  An important detail to consider when using buttons is that buttons
	//  physically bounce when you press and release them.  Typically this
	//  bouncing is on the order of 1 to 10 ms, and if you're not careful
	//  it can trick you into thinking that a pressed button has been
	//  released when it hasn't.  To get around this, you should always
	//  "debounce" your buttons.  One simple way to do this is to igore the
	//  button state for around 10 ms after you first notice it change.  In
	//  this program, we debounce the buttons using a call to _delay_ms,
	//  which comes from the file <util.delay>.  Note that _delay_ms() can
	//  only delay for a maximum of around 13 ms with an IO clock of 20 MHz.

	//  Note that the button routines in the Pololu AVR library perform this
	//  debouncing for you and provide a delay_ms() function that does not
	//  have this 13 ms limit.


	// ***** BUTTON INITIALIZATION *****

	// set button pins as inputs (this is their default state, so the following
	//  line isn't really necessary)
	DDRB &= ~(1 << BUTTON0) & ~(1 << BUTTON1) & ~(1 << BUTTON2);

	// enable pull-ups on the button pins
	PORTB |= (1 << BUTTON0) | (1 << BUTTON1) | (1 << BUTTON2);


	// ***** BUZZER INITIALIZATION *****

	DDRB &= ~(1 << PB2);	// set buzzer pin as an input to disable the buzzer

	// configure the timer 1 hardware module to generate a PWM on the buzzer
	//  pin (PB2):

	TCCR1A = 0x23;	// bits 6 and 7 clear: normal port op., OC1A disconnected
					// bit 4 clear, 5 set: clear OC1B (PB2) on compare match
					// bits 2 and 3: not used
					// bits 0 & 1 set: combine with bits 3 & 4 of TCCR1B...

	TCCR1B = 0x19;	// bit 5, 6, 7: not used
					// bits 3 and 4 set: combine with bits 0 & 1 of TCCR1A to
					// 		select waveform generation mode 14, fast PWM,
					//		TOP = OCR1A, compare match = OCR1B
					// bit 0 set, 1&2 clear: timer clock = IO clk (prescaler 1)

	// We have now set up a PWM on pin PB2 that will only affect the buzzer
	//  when the buzzer pin (PB2) is set as an output.  The buzzer PWM uses
	//  a 16-bit timer whose source is the IO clock (20 MHz).  It
	//  drives pin PB2 high when it overflows (i.e. when the timer 1 count
	//  TCNT1 equals the value in register OCR1A) and it drives pin PB2 low
	//  when a compare match occurs (i.e. when the timer 1 count TCNT1 equals
	//  the value in register OCR1B).  Therefore, OCR1A determines the
	//  frequency of the PWM and OCR1B determines the duty cycle of the PWM.
	//
	//  buzzer frequency = timer 1 frequency / PWM TOP
	//                   = IO clk / (OCR1A + 1)
	//                   = 20 MHz / (OCR1A + 1)
	//  so, OCR1A = (20 MHz / desired frequency) - 1
	//  duty cycle = OCR1B / (OCR1A + 1)

	
	while (1)		// loop forever
	{
		DDRB &= ~(1 << PB2);	// disable buzzer (buzzer pin -> input)
		OCR1B = 0;				// set buzzer duty cycle to its minimum value

		_delay_ms( 10 );		// loop here for 10 ms to give the button
								//  signal time to stop bouncing

		if ( (PINB & (1 << BUTTON0)) == 0 )
		{
			_delay_ms( 10 );	// loop here for 10 ms to give the button
								//  signal time to stop bouncing
			DDRB |= 1 << PB2;	// enable buzzer by setting pin as an output
			OCR1A = 9999;		// set buzzer frequency = 2 kHz
			OCR1B = OCR1A >> 1;	// make OCR1B half of OCR1A (50% duty cycle)

			// loop while the button is held down
			while ( (PINB & (1 << BUTTON0)) == 0 )
				;
		}

		else if ( (PINB & (1 << BUTTON1)) == 0 )
		{
			_delay_ms( 10 );	// loop here for 10 ms to give the button
								//  signal time to stop bouncing
			DDRB |= 1 << PB2;	// enable buzzer by setting pin as an output
			OCR1A = 19999;		// set buzzer frequency = 1 kHz
			OCR1B = OCR1A >> 1;	// make OCR1B half of OCR1A (50% duty cycle)

			// loop while the button is held down
			while ( (PINB & (1 << BUTTON1)) == 0 )
				;
		}

		else if ( (PINB & (1 << BUTTON2)) == 0 )
		{
			_delay_ms( 10 );	// loop here for 10 ms to give the button
								//  signal time to stop bouncing
			DDRB |= 1 << PB2;	// enable buzzer by setting pin as an output
			OCR1A = 39999;		// set buzzer frequency = 500 Hz
			OCR1B = OCR1A >> 1;	// make OCR1B half of OCR1A (50% duty cycle)

			// loop while the button is held down
			while ( (PINB & (1 << BUTTON2)) == 0 )
				;
		}
	}

	return 0;
}
